package OAIGet;
#
# OAI download routine that respects 503 RetryAfter returns
#  and 302 redirect returns.
#
# Written with libwww-perl-5.48 and perl 5.005_03
# Simeon Warner - 14Sep2000...
#
# [CVS: $Id: OAIGet.pm,v 1.1 2001/06/05 20:34:23 simeon Exp $]
use strict;
use vars qw(@ISA @EXPORT);
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(&OAIGet);

use LWP::UserAgent;

my $myname='OAIGet';
my ($version)='$Revision: 1.1 $'=~/([\d\.]+)/;
my $DefaultUserAgent="$myname-$version";

my $maxTries=10;
my $defaultRetryAfter=60;
my $VERBOSE=0;

# Parmaters:
#   baseURL     is URL of OAI interface script
#   args        hash of arg=val pairs
#   type        GET or POST
#   contact     email address to be sent with HTTP request
#   userAgent   User-Agent string for request
#   verbose     
#
# Returns:
#   code        undef for ok, HTTP or OAIGet error code otherwise
#   msg         message that goes with the code
#   content     content returned
#
# 9xx codes are returned to indicate errors in this script
#
sub OAIGet {
  my ($baseURL,$args,$type,$contact,$userAgent,$numRedirect,$verbose)=@_;

  $VERBOSE=1 if ($verbose);
  unless (defined($contact) and $contact=~/@/) {
    return(900,"$myname error - bad contact parameter, should be email");
  }
  
  unless (defined($userAgent)) { $userAgent=$DefaultUserAgent; }
  my $ua = new LWP::UserAgent;
  $ua->agent($userAgent);

  my $avpairs=undef;
  if (defined(%$args)) {
    foreach my $arg (keys %$args) {
      if (defined($avpairs)) { $avpairs.='&'; } else { $avpairs=''; }
      $avpairs.=$arg.'='.$args->{$arg};
    }
  }

  # Create a request
  my $req;
  &vprint("Doing $type to $baseURL args: $avpairs\n");
  if ($type eq 'GET') {
    $req = new HTTP::Request(GET => $baseURL.'?'.$avpairs);
  } elsif ($type eq 'POST') {
    $req = new HTTP::Request(POST => $baseURL);
    $req->content_type('application/x-www-form-urlencoded');
    $req->content($avpairs);
  } else {
    return(901,"$myname error - type must be GET or POST",'');
  }
  $req->from($contact);

  my $numTries=0;
  while ($numTries<$maxTries) {
    $numTries++;
    my $res = $ua->request($req);

    if ($res->code==503) {
      my $sleep = $res->header('Retry-After');
      if (not defined($sleep)) {
        $sleep=$defaultRetryAfter;
        &vprint("Warning - 503 but no Retry-After, using ${sleep}s.\n");
        &vprintHeaders($res);
      } elsif (($sleep<0) or ($sleep>86400)) {
        return(902,"$myname error - bad Retry-After time ($sleep)");
      }
      &vprint("Got 503, sleeping for $sleep seconds...\n");
      sleep($sleep);
      &vprint("Woken again, retrying...\n");
    } elsif ($res->code==302) {
      my $redirectURL=$res->header('Location');
      my ($redirectBaseURL)=$redirectURL=~/^([^\?]+)\?/;
      if (not defined($redirectURL)) {
        &vprintHeaders($res);
        return(903,"$myname error - 302 with no redirect");
      } elsif ($numRedirect<0) {
        return(904,"$myname error - 302 but redirect not permitted");
      } elsif ((++$numRedirect)>5) {
        return(905,"$myname error - too many redirects! ($numRedirect)");
      } 
      &vprint("Got 302, redirecting to $redirectURL...\n");
      return(&OAIGet($redirectBaseURL,$args,$type,$contact,$userAgent,$numRedirect,$verbose));
    } else {
      # return response
      &vprint("Got ",$res->code," ",$res->message,
           " (",length($res->content),"bytes)\n");
      return($res->code,$res->message,$res->content);
    }
  }
  return(905,"$myname error - too many retries! ($numTries)");
}

sub vprint {
  print "$myname: ",@_ if ($VERBOSE);
}
 
sub vprintHeaders {
  my ($res)=@_;
  return unless ($VERBOSE);
  print "$myname: Headers:\n";
  my $h=$res->headers_as_string();
  $h=~s/\s+$//;
  $h=~s/\n/\n  /g;
  print "  $h\n";
}

1;
